-- List and breakdown of spells is taken from PitBull_Aura

local Module = UnderHood:GetModule("Auras")

local error = error
local GetSpellInfo = GetSpellInfo
local GetTalentInfo = GetTalentInfo
local pairs = pairs
local strformat = string.format
local twipe = table.wipe

local canDispel, friendBuffs, friendDebuffs, selfBuffs, selfDebuffs, petBuffs, petDebuffs, enemyDebuffs
local totalSelfBuffs, totalPetBuffs, totalSelfDebuffs, totalPetDebuffs, allFriendlyBuffs, allFriendlyDebuffs
local _, playerClass = UnitClass("player")
local _, playerRace = UnitRace("player")

local function scanForKnownTalent(spellid) 
	local wantedName, _ = GetSpellInfo(spellid)
	if not wantedName then return nil end
	local numTabs = GetNumTalentTabs()
	for t=1, numTabs do
		local numTalents = GetNumTalents(t)
		for i=1, numTalents do
			local nameTalent, iconPath, tier, column, currentRank = GetTalentInfo(t, i)
			if nameTalent and (nameTalent == wantedName) then
				if currentRank and (currentRank > 0) then
					return true
				else
					return nil
				end
			end
		end
	end
	return nil
end

if playerClass == "DEATHKNIGHT" then
	canDispell = {}
	selfBuffs = {
		[49222] = true, -- Bone Shield
		[51789] = true, -- Blade Barrier
		[64855] = true, -- Blade Barrier
		[64856] = true, -- Blade Barrier
		[81141] = true, -- Crimson Scourge
		[49028] = true, -- Dancing Rune Weapon
		[59052] = true, -- Freezing Fog (Rime)
		[51124] = true, -- Killing Machine
		[49039] = true, -- Lichborne
		[48792] = true, -- Icebound Fortitude
		[51271] = true, -- Pillar of Frost
		[50421] = true, -- Scent of Blood
		[81340] = true, -- Sudden Doom
		[49194] = true, -- Unholy Blight
		[55233] = true, -- Vampiric Blood
		[81162] = true, -- Will of the Necropolis
	}
	selfDebuffs = {}
	friendBuffs = {
		[63560] = true, -- Dark Transformation
		[57330] = true, -- Horn of Winter
		[3714]  = true, -- Path of Frost
		[49016] = true, -- Unholy Frenzy
	}
	friendDebuffs = {}
	petBuffs = {
		[19705] = true, -- Well Fed
		[91342] = true, -- Shadow Infusion
	}
	enemyDebuffs = {
		[55078] = true, -- Blood Plague
		[81325] = true, -- Brittle Bones
		[81326] = true, -- Brittle Bones
		[50434] = true, -- Chilblains
		[50435] = true, -- Chilblains
		[45524] = true, -- Chains of Ice
		[55095] = true, -- Frost Fever
		[49203] = true, -- Hungering Cold
		[73975] = true, -- Necrotic Strike
		[96265] = true, -- Scarlet Fever
		[81130] = true, -- Scarlet Fever
		[47476] = true, -- Strangulate
		[49206] = true, -- Summon Gargoyle
	}
elseif playerClass == "DRUID" then
	canDispel = {
		["Curse"] = true,
		["Poison"] = true,
		["Magic"] = scanForKnownTalent(88423),
	}
	selfBuffs = {
		[1066]  = true, -- Aquatic Form
		[22812] = true, -- Barkskin
		[50344] = true, -- Berserk
		[5487]  = true, -- Bear Form
		[768]   = true, -- Cat Form
		[16870] = true, -- Clearcasting
		[1850]  = true, -- Dash
		[5229]  = true, -- Enrage
		[34151] = true, -- Efflorescence
		[81274] = true, -- Efflorescence
		[81275] = true, -- Efflorescence
		[81262] = true, -- Efflorescence
		[33943] = true, -- Flight Form
		[22842] = true, -- Frenzied Regeneration
		[81093] = true, -- Fury of Stormrage
		[81006] = true, -- Lunar Shower
		[81191] = true, -- Lunar Shower
		[81192] = true, -- Lunar Shower
		[24858] = true, -- Moonkin Form
		[16880] = true, -- Nature's Grace
		[16689] = true, -- Nature's Grasp
		[17116] = true, -- Nature's Swiftness
		[16864] = true, -- Omen of Clarity
		[48391] = true, -- Owlkin Frenzy
		[69369] = true, -- Predator's Swiftness
		[5215]  = true, -- Prowl
		[62606] = true, -- Savage Defense
		[52610] = true, -- Savage Roar
		[93400] = true, -- Shooting Stars
		[48505] = true, -- Starfall
		[50322] = true, -- Survival Instincts
		[40120] = true, -- Swift Flight Form
		[5217]  = true, -- Tiger's Fury
		[5225]  = true, -- Track Humanoids
		[783]   = true, -- Travel Form
	}
	friendBuffs = {
		[29166] = true, -- Innervate
		[17007] = true, -- Leader of the Pack
		[33763] = true, -- Lifebloom
		[48504] = true, -- Living Seed
		[1126]  = true, -- Mark of the Wild: alt 79060
		[24907] = true, -- Moonkin Aura
		[8936]  = true, -- Regrowth
		[774]   = true, -- Rejuvenation
		[77761] = true, -- Stampeding Roar
		[77764] = true, -- Stampeding Roar
		[467]   = true, -- Thorns
		[740]   = true, -- Tranquility
		[5420]  = true, -- Tree of Life
		[48438] = true, -- Wild Growth
	}
	enemyDebuffs = {
		[5211]  = true, -- Bash
		[5209]  = true, -- Challenging Roar
		[33786] = true, -- Cyclone
		[99]    = true, -- Demoralizing Roar
		[339]   = true, -- Entangling Roots
		[60433] = true, -- Earth and Moon
		[770]   = true, -- Faerie Fire
		[16857] = true, -- Faerie Fire (Feral)
		[45334] = true, -- Feral Charge (Bear)
		[50259] = true, -- Feral Charge (Cat)
		[81288] = true, -- Fungal Growth
		[81281] = true, -- Fungal Growth
		[2637]  = true, -- Hibernate
		[16914] = true, -- Hurricane
		[58179] = true, -- Infected Wounds
		[58180] = true, -- Infected Wounds
		[5570]  = true, -- Insect Swarm
		[33745] = true, -- Lacerate
		[22570] = true, -- Maim
		[33878] = true, -- Mangle (Bear)
		[33876] = true, -- Mangle (Cat)
		[8921]  = true, -- Moonfire
		[9005]  = true, -- Pounce
		[9007]  = true, -- Pounce Bleed Effect
		[1822]  = true, -- Rake
		[1079]  = true, -- Rip
		[93402] = true, -- Sunfire
		[77758] = true, -- Thrash
	}
elseif playerClass == "HUNTER" then
	canDispel = {
		["Magic"] = true,
		["Enrage"] = true,
	}
	selfBuffs = {
		[5118]  = true, -- Aspect of the Cheetah
		[13165] = true, -- Aspect of the Hawk
		[19263] = true, -- Deterrence
		[6197]  = true, -- Eagle Eye
		[5384]  = true, -- Feign Death
		[34506] = true, -- Master Tactician
		[6150]  = true, -- Quick Shots
		[3045]  = true, -- Rapid Fire
		[34948] = true, -- Rapid Killing
		[34471] = true, -- The Beast Within
	}
	friendBuffs = {
		[13159] = true, -- Aspect of the Pack
		[20043] = true, -- Aspect of the Wild
		[75447] = true, -- Ferocious Inspiration
		[34477] = true, -- Misdirection
		[19578] = true, -- Spirit Bond
		[19506] = true, -- Trueshot Aura
		[57669] = true, -- Replenishment
		[54216] = true, -- Master's Call
	}
	enemyDebuffs = {
		[19434] = true, -- Aimed Shot
		[1462]  = true, -- Beast Lore
		[3385]  = true, -- Boar Charge
		[35100] = true, -- Concussive Barrage
		[5116]  = true, -- Concussive Shot
		[19306] = true, -- Counterattack
		[3408]  = true, -- Crippling Poison
		[2818]  = true, -- Deadly Poison
		[19184] = true, -- Entrapment
		[13812] = true, -- Explosive Trap Effect
		[7140]  = true, -- Expose Weakness
		[34889] = true, -- Fire Breath
		[1543]  = true, -- Flare
		[3355]  = true, -- Freezing Trap Effect
		[13810] = true, -- Frost Trap Aura
		[1853]  = true, -- Growl
		[1130]  = true, -- Hunter's Mark
		[7093]  = true, -- Intimidation
		[5760]  = true, -- Mind-numbing Poison
		[32093] = true, -- Poison Spit
		[1513]  = true, -- Scare Beast
		[19503] = true, -- Scatter Shot
		[6411]  = true, -- Scorpid Poison
		[24423] = true, -- Screech
		[1978]  = true, -- Serpent Sting
		[34490] = true, -- Silencing Shot
		[1515]  = true, -- Tame Beast
		[2974]  = true, -- Wing Clip
		[19386] = true, -- Wyvern Sting
	}
	petBuffs = {
		[19574] = true, -- Bestial Wrath
		[3385]  = true, -- Boar Charge
		[1850]  = true, -- Dash
		[23145] = true, -- Dive
		[1539]  = true, -- Feed Pet Effect
		[19451] = true, -- Frenzy
		[3149]  = true, -- Furious Howl
		[136]   = true, -- Mend Pet
		[5215]  = true, -- Prowl
		[26064] = true, -- Shell Shield
		[32920] = true, -- Warp
		[19705] = true, -- Well Fed
	}
elseif playerClass == "MAGE" then
	canDispel = {
		["Curse"] = true,
	}
	selfBuffs = {
		[31571] = true, -- Arcane Potency
		[12042] = true, -- Arcane Power
		[31641] = true, -- Blazing Speed
		[1953]  = true, -- Blink
		[12536] = true, -- Clearcasting
		[11129] = true, -- Combustion
		[12051] = true, -- Evocation
		[57761] = true, -- Fireball! (instant cast fireball proc from Brain Freeze)
		[54741] = true, -- Firestarter
		[44440] = true, -- Firey Payback
		[7302]  = true, -- Frost Armor
		[44445] = true, -- Hot Streak
		[7302]  = true, -- Ice Armor
		[11426] = true, -- Ice Barrier
		[27619] = true, -- Ice Block
		[44394] = true, -- Incanter's Absorption
		[66]    = true, -- Invisibility
		[6117]  = true, -- Mage Armor
		[1463]  = true, -- Mana Shield
		[30482] = true, -- Molten Armor
		[12043] = true, -- Presence of Mind
	}
	selfDebuffs = {
		[10833] = true, -- Arcane Blast
		[41425] = true, -- Hypothermia
	}
	friendBuffs = {
		[79057] = true, -- Arcane Brilliance
		[1459]  = true, -- Arcane Intellect
		[79039] = true, -- Dalaran Brilliance
		[54648] = true, -- Focus Magic
		[130]   = true, -- Slow Fall
		[57669] = true, -- Replenishment
	}
	enemyDebuffs = {
		[11113]	= true, -- Blast Wave
		[10]    = true, -- Blizzard
		[6136]  = true, -- Chilled
		[120]   = true, -- Cone of Cold
		[29964] = true, -- Dragon's Breath
		[22959] = true, -- Fire Vulnerability
		[133]   = true, -- Fireball
		[2120]  = true, -- Flamestrike
		[122]   = true, -- Frost Nova
		[116]   = true, -- Frostbolt
		[7302]  = true, -- Ice Armor
		[3261]  = true, -- Ignite
		[11103] = true, -- Impact
		[44457] = true, -- Living Bomb
		[118]   = true, -- Polymorph
		[11366] = true, -- Pyroblast
		[246]   = true, -- Slow
		[11180] = true, -- Winter's Chill
	}
elseif playerClass == "PALADIN" then
	canDispel = {
		["Magic"] = scanForKnownTalent(53551),
		["Poison"] = true,
		["Disease"] = true,
	}
	selfBuffs = {
		[31884] = true, -- Avenging Wrath
		[54428]	= true, -- Divine Plea
		[498]   = true, -- Divine Protection
		[642]   = true, -- Divine Shield
		[9800]  = true, -- Holy Shield
		[54149]	= true, -- Infusion of Light
		[20178]	= true, -- Reckoning
		[25780] = true, -- Righteous Fury
		[20164] = true, -- Seal of Justice
		[20165] = true, -- Seal of Insight
		[20154] = true, -- Seal of Righteousness
		[31801] = true, -- Seal of Truth
		[5502]  = true, -- Sense Undead
		[23214] = true, -- Summon Charger
		[13819] = true, -- Summon Warhorse
		[8602]  = true, -- Vengeance
	}
	friendBuffs = {
		[64364]	= true, -- Aura Mastery
		[53563] = true, -- Beacon of Light
		[79062] = true, -- Blessing of Kings
		[79101] = true, -- Blessing of Might
		[19746] = true, -- Concentration Aura
		[32223] = true, -- Crusader Aura
		[465]   = true, -- Devotion Aura
		[19891] = true, -- Resistance Aura
		[1044]  = true, -- Hand of Freedom
		[1022]  = true, -- Hand of Protection
		[6940]  = true, -- Hand of Sacrifice
		[7294]  = true, -- Retribution Aura
		[57669] = true, -- Replenishment
	}
	enemyDebuffs = {
		[31935] = true, -- Avenger's Shield
		[20116] = true, -- Consecration
		[853]   = true, -- Hammer of Justice
		[31803] = true, -- Holy Vengeance
		[20066] = true, -- Repentance
		[25]    = true, -- Stun, from Seal of Justice
		[10326] = true, -- Turn Evil
		[26017]	= true, -- Vindication
	}
	friendDebuffs = {
		[25771] = true, -- Forbearance
	}
elseif playerClass == "PRIEST" then
	canDispel = {
		["Curse"] = true,
		["Disease"] = true,
		["Poison"] = true,
	}
	selfBuffs = {
		[27811] = true, -- Blessed Recovery
		[33143]	= true, -- Blessed Resilience
		[59887]	= true, -- Borrowed Time
		[47585] = true, -- Dispersion
		[586]   = true, -- Fade
		[45241]	= true, -- Focused Will
		[45242]	= true, -- Focused Will
		[34753]	= true, -- Holy Concentration
		[588]   = true, -- Inner Fire
		[14751] = true, -- Inner Focus
		[2096]  = true, -- Mind Vision
		[63731]	= true, -- Serendipity
		[15473] = true, -- Shadowform
		[27827] = true, -- Spirit of Redemption
		[33151]	= true, -- Surge of Light
		[91724] = true, -- Spell Warding
	}
	friendBuffs = {
		[6346]  = true, -- Fear Ward
		[56161] = true, -- Glyph of Prayer of Healing
		[47930]	= true, -- Grace
		[47788]	= true, -- Guardian Spirit
		[14892] = true, -- Inspiration
		[1706]  = true, -- Levitate
		[7001]	= true, -- Lightwell Renew
		[79104] = true, -- Power Word: Fortitude
		[17]    = true, -- Power Word: Shield
		[33206] = true, -- Pain Suppression
		[33076] = true, -- Prayer of Mending
		[139]   = true, -- Renew
		[79106] = true, -- Shadow Protection
		[57669] = true, -- Replenishment
	}
	enemyDebuffs = {
		[2944]  = true, -- Devouring Plague
		[14914] = true, -- Holy Fire
		[605]   = true, -- Mind Control
		[15407] = true, -- Mind Flay
		[48045]	= true, -- Mind Sear
		[453]   = true, -- Mind Soothe
		[2096]  = true, -- Mind Vision
		[33196]	= true, -- Misery
		[64058] = true, -- Psychic Horror
		[8122]  = true, -- Psychic Scream
		[9484]  = true, -- Shackle Undead
		[589]   = true, -- Shadow Word: Pain
		[6726]  = true, -- Silence
		[15286] = true, -- Vampiric Embrace
		[34914] = true, -- Vampiric Touch
	}
	friendDebuffs = {
		[2096]  = true, -- Mind Vision
		[6788]  = true, -- Weakened Soul
	}
elseif playerClass == "ROGUE" then
	selfBuffs = {
		[13750] = true, -- Adrenaline Rush
		[13877] = true, -- Blade Flurry
		[31224] = true, -- Cloak of Shadows
		[14177] = true, -- Cold Blood
		[4086]  = true, -- Evasion
		[14143] = true, -- Remorseless
		[36554] = true, -- Shadowstep
		[5171]  = true, -- Slice and Dice
		[2983]  = true, -- Sprint
		[1784]  = true, -- Stealth
		[1856]  = true, -- Vanish
	}
	friendBuffs = {
		[57934] = true, -- Tricks of the Trade
	}
	enemyDebuffs = {
		[2094]  = true, -- Blind
		[1833]  = true, -- Cheap Shot
		[3408]  = true, -- Crippling Poison
		[2818]  = true, -- Deadly Poison
		[26679] = true, -- Deadly Throw
		[8647]  = true, -- Expose Armor
		[703]   = true, -- Garrote
		[1330]  = true, -- Garrote - Silence
		[1776]  = true, -- Gouge
		[16511] = true, -- Hemorrhage
		[18425] = true, -- Kick - Silenced
		[408]   = true, -- Kidney Shot
		[5760]  = true, -- Mind-numbing Poison
		[14251] = true, -- Riposte
		[1943]  = true, -- Rupture
		[6770]  = true, -- Sap
		[13218] = true, -- Wound Poison
	}
elseif playerClass == "SHAMAN" then
	canDispel = {
		["Curse"] = true,
		["Magic"] = scanForKnownTalent(77130),
	}
	selfBuffs = {
		[52179] = true, -- Astral Shift
		[16246] = true, -- Clearcasting
		[52127] = true, -- Elemental Devastation
		[64701] = true, -- Elemental Mastery
		[6196]  = true, -- Far Sight
		[2645]  = true, -- Ghost Wolf
		[324]   = true, -- Lightning Shield
		[53817] = true, -- Maelstrom Weapon
		[16188] = true, -- Nature's Swiftness
		[43339] = true, -- Shamanistic Focus (Focused)
		[30823] = true, -- Shamanistic Rage
		[53390] = true, -- Tidal Waves
		[52127] = true, -- Water Shield
		[16257]	= true, -- Flurry
		[58875]	= true, -- Spirit Walk
	}
	friendBuffs = {
		[16177] = true, -- Ancestral Fortitude
		[2825]  = playerRace == "Troll" or playerRace == "Tauren" or playerRace == "Orc", -- Bloodlust
		[379]   = true, -- Earth Shield
		[51945] = true, -- Earthliving
		[4057]  = true, -- Fire Resistance
		[8227]  = true, -- Flametongue Totem
		[4077]  = true, -- Frost Resistance
		[8178]  = true, -- Grounding Totem Effect
		[5672]  = true, -- Healing Stream
		[29202] = true, -- Healing Way
		[23682] = playerRace == "Draenei", -- Heroism
		[5677]  = true, -- Mana Spring
		[16191] = true, -- Mana Tide
		[4081]  = true, -- Nature Resistance
		[61295] = true, -- Riptide
		[8072]  = true, -- Stoneskin
		[8076]  = true, -- Strength of Earth
		[77747] = true, -- Totem of Wrath
		[131]   = true, -- Water Breathing
		[546]   = true, -- Water Walking
		[27621] = true, -- Windfury Totem
		[2895]  = true, -- Wrath of Air Totem
	}
	friendDebuffs = {
		[57723] = playerRace == "Draenei", -- Exhaustion
		[57724] = playerRace == "Troll" or playerRace == "Tauren" or playerRace == "Orc", -- Sated
	}
	enemyDebuffs = {
		[3600]  = true, -- Earthbind
		[8050]  = true, -- Flame Shock
		[8056]  = true, -- Frost Shock
		[8034]  = true, -- Frostbrand Attack
		[39796] = true, -- Stoneclaw Stun
		[17364] = true, -- Stormstrike
		[51514]	= true, -- Hex
		[58861]	= true, -- Bash
	}
	petBuffs = {
		[58875]	= true, -- Spirit Walk
	}
elseif playerClass == "WARLOCK" then
	canDispel = {
		["Magic"] = true,
	}
	selfBuffs = {
		[34935] = true, -- Backlash
		[687]   = true, -- Demon Armor
		[28176] = true, -- Fel Armor
		[30299] = true, -- Nether Protection
		[1050]  = true, -- Sacrifice
		[5500]  = true, -- Sense Demons
		[17941] = true, -- Shadow Trance
		[6229]  = true, -- Shadow Ward
		[19028] = true, -- Soul Link
		[23161] = true, -- Summon Dreadsteed
		[1710]  = true, -- Summon Felsteed
		[47241]	= true, -- Metamorphosis
	}
	friendBuffs = {
		[6307]  = true, -- Blood Pact
		[134]   = true, -- Fire Shield
		[54424] = true, -- Fel Intelligence
		[20707] = true, -- Soulstone Resurrection
		[5697]  = true, -- Unending Breath
		[57669] = true, -- Replenishment
	}
	enemyDebuffs = {
		[18118] = true, -- Aftermath
		[710]   = true, -- Banish
		[172]   = true, -- Corruption
		[89]    = true, -- Cripple
		[980]   = true, -- Curse of Agony
		[603]   = true, -- Curse of Doom
		[18223] = true, -- Curse of Exhaustion
		[1714]  = true, -- Curse of Tongues
		[702]   = true, -- Curse of Weakness
		[1490]  = true, -- Curse of the Elements
		[6789]  = true, -- Death Coil
		[689]   = true, -- Drain Life
		[5138]  = true, -- Drain Mana
		[1120]  = true, -- Drain Soul
		[5782]  = true, -- Fear
		[1949]  = true, -- Hellfire
		[5484]  = true, -- Howl of Terror
		[348]   = true, -- Immolate
		[1122]  = true, -- Inferno
		[4629]  = true, -- Rain of Fire
		[6358]  = true, -- Seduction
		[27243] = true, -- Seed of Corruption
		[17877] = true, -- Shadowburn
		[30283] = true, -- Shadowfury
		[6726]  = true, -- Silence
		[6360]  = true, -- Soothing Kiss
		[24259] = true, -- Spell Lock
		[17735] = true, -- Suffering
		[54049] = true, -- Shadow Bite
		[47960] = true, -- Shadowflame
		[30108] = true, -- Unstable Affliction
	}
	petBuffs = {
		[23257] = true, -- Demonic Frenzy
		[19705] = true, -- Well Fed
	}
elseif playerClass == "WARRIOR" then
	selfBuffs = {
		[18499] = true, -- Berserker Rage
		[16487] = true, -- Blood Craze
		[23880] = true, -- Bloodthirst
		[3019]  = true, -- Enrage
		[55694]	= true, -- Enraged Regeneration
		[12319] = true, -- Flurry
		[12975] = true, -- Last Stand
		[8285]  = true, -- Rampage
		[1719]  = true, -- Recklessness
		[20230] = true, -- Retaliation
		[15604] = true, -- Second Wind
		[2565]  = true, -- Shield Block
		[871]   = true, -- Shield Wall
		[9941]  = true, -- Spell Reflection
		[12328] = true, -- Sweeping Strikes
		-- T4, Tank, 2/4 piece bonus
		[37514] = true, -- Blade Turning
		[6572]  = true, -- Revenge
		-- T5, Tank, 2/4 piece bonus
		[37525] = true, -- Battle Rush
		[37523] = true, -- Reinforced Shield
		-- T5, DPS, 2 piece bonus
		[7384]  = true, -- Overpower
	}
	selfDebuffs = {
		[12292] = true, -- Death Wish
	}
	friendBuffs = {
		[6673]  = true, -- Battle Shout
		[469]   = true, -- Commanding Shout
		[3411]  = true, -- Intervene
		[50720] = true, -- Vigilance
	}
	enemyDebuffs = {
		[16952] = true, -- Blood Frenzy
		[1161]  = true, -- Challenging Shout
		[7922]  = true, -- Charge Stun
		[12809] = true, -- Concussion Blow
		[1604]  = true, -- Dazed
		[12721] = true, -- Deep Wound
		[1160]  = true, -- Demoralizing Shout
		[676]   = true, -- Disarm
		[1715]  = true, -- Hamstring
		[12289] = true, -- Improved Hamstring
		[20253] = true, -- Intercept Stun
		[5246]  = true, -- Intimidating Shout
		[9347]  = true, -- Mortal Strike
		[10576] = true, -- Piercing Howl
		[772]   = true, -- Rend
		[18498] = true, -- Shield Bash - Silenced
		[7386]  = true, -- Sunder Armor
		[355]   = true, -- Taunt
		[6343]  = true, -- Thunder Clap
	}
end

if not canDispel then
	canDispel = {}
end
if not selfBuffs then
	selfBuffs = {}
end
if not selfDebuffs then
	selfDebuffs = {}
end
if not friendBuffs then
	friendBuffs = {}
end
if not enemyDebuffs then
	enemyDebuffs = {}
end
if not friendDebuffs then
	friendDebuffs = {}
end
if not petBuffs then
	petBuffs = {}
end
if not petDebuffs then
	petDebuffs = {}
end
if playerRace == "BloodElf" then
	enemyDebuffs[28730] = true -- Arcane Torrent
	enemyDebuffs[50613] = true -- Arcane Torrent
	enemyDebuffs[80483] = true -- Arcane Torrent
	enemyDebuffs[25046] = true -- Arcane Torrent
--	enemyDebuffs[69179] = true -- Arcane Torrent - No BE Warrior yet.
elseif playerRace == "Troll" then
	selfBuffs[26297] = true -- Berserking
elseif playerRace == "Orc" then
	selfBuffs[20572] = true -- Blood Fury
	selfBuffs[33702] = true -- Blood Fury
	selfBuffs[33697] = true -- Blood Fury
elseif playerRace == "Scourge" then
	selfBuffs[20578] = true -- Cannibalize
elseif playerRace == "Tauren" then
	enemyDebuffs[20549] = true -- War Stomp
elseif playerRace == "Dwarf" then
	selfBuffs[20594] = true -- Stoneform
elseif playerRace == "Draenei" then
	friendBuffs[28880] = true -- Gift of the Naaru
	friendBuffs[59547] = true -- Gift of the Naaru
	friendBuffs[59544] = true -- Gift of the Naaru
	friendBuffs[59542] = true -- Gift of the Naaru
	friendBuffs[59548] = true -- Gift of the Naaru
	friendBuffs[59543] = true -- Gift of the Naaru
	friendBuffs[59545] = true -- Gift of the Naaru
elseif playerRace == "NightElf" then
	selfBuffs[58984] = true -- New Shadowmeld in Wrath
elseif playerRace == "Worgen" then
	selfBuffs[68992] = true -- Darkflight
	selfBuffs[87840] = true -- Running Wild
end
if playerRace == "BloodElf" or playerRace == "Troll" or playerRace == "Orc" or playerRace == "Scourge" or playerRace == "Tauren" then
	-- horde
	friendBuffs[23335] = true -- Silverwing Flag
else
	-- alliance
	friendBuffs[23333] = true -- Warsong Flag
end
friendBuffs[34976] = true -- Netherstorm Flag

local function turn(t)
	local tmp = {}

	for k in pairs(t) do
		local spell = GetSpellInfo(k)
		if not spell then
			error(strformat("Unknown spell ID: %d", k))
		end
		tmp[spell] = true
		t[k] = nil
	end
	for k in pairs(tmp) do
		t[k] = true
		tmp[k] = nil
	end
	tmp = twipe(tmp)
end
turn(friendBuffs)
turn(friendDebuffs)
turn(selfBuffs)
turn(selfDebuffs)
turn(petBuffs)
turn(petDebuffs)
turn(enemyDebuffs)

totalSelfBuffs, totalPetBuffs, totalSelfDebuffs, totalPetDebuffs, allFriendlyBuffs, allFriendlyDebuffs = {}, {}, {}, {}, {}, {}
for k in pairs(selfBuffs) do
	totalSelfBuffs[k] = true
	allFriendlyBuffs[k] = true
end
for k in pairs(selfDebuffs) do
	totalSelfDebuffs[k] = true
	allFriendlyDebuffs[k] = true
end
for k in pairs(petBuffs) do
	totalPetBuffs[k] = true
	allFriendlyBuffs[k] = true
end
for k in pairs(petDebuffs) do
	totalPetDebuffs[k] = true
	allFriendlyDebuffs[k] = true
end
for k in pairs(friendBuffs) do
	totalSelfBuffs[k] = true
	totalPetBuffs[k] = true
	allFriendlyBuffs[k] = true
end
for k in pairs(friendDebuffs) do
	totalSelfDebuffs[k] = true
	totalPetDebuffs[k] = true
	allFriendlyDebuffs[k] = true
end

Module.Spells = {
	canDispel = canDispel,
	friendBuffs = friendBuffs,
	friendDebuffs = friendDebuffs,
	selfDebuffs = selfDebuffs,
	petBuffs = petBuffs,
	petDebuffs = petDebuffs,
	enemyDebuffs = enemyDebuffs,
	totalSelfBuffs = totalSelfBuffs,
	totalPetBuffs = totalPetBuffs,
	totalSelfDebuffs = totalSelfDebuffs,
	totalPetDebuffs = totalPetDebuffs,
	allFriendlyBuffs = allFriendlyBuffs,
	allFriendlyDebuffs = allFriendlyDebuffs,
}
